from __future__ import unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible

from nodeconductor.core.models import RuntimeStateMixin, StateMixin
from nodeconductor.quotas.fields import CounterQuotaField
from nodeconductor.quotas.models import QuotaModelMixin
from nodeconductor.structure import models as structure_models


from .log import alert_logger


class DigitalOceanService(structure_models.Service):
    projects = models.ManyToManyField(
        structure_models.Project, related_name='digitalocean_services', through='DigitalOceanServiceProjectLink')

    class Meta(structure_models.Service.Meta):
        verbose_name = 'DigitalOcean service'
        verbose_name_plural = 'DigitalOcean services'

    class Quotas(QuotaModelMixin.Quotas):
        droplet_count = CounterQuotaField(
            default_limit=50,
            target_models=lambda: [Droplet],
            path_to_scope='service_project_link.service'
        )

    def raise_readonly_token_alert(self):
        """ Raise alert if provided token is read-only """
        alert_logger.digital_ocean.warning(
            'DigitalOcean token for {settings_name} is read-only.',
            scope=self.settings,
            alert_type='token_is_read_only',
            alert_context={'settings': self.settings})

    def close_readonly_token_alert(self):
        alert_logger.digital_ocean.close(scope=self.settings, alert_type='token_is_read_only')

    @classmethod
    def get_url_name(cls):
        return 'digitalocean'


class DigitalOceanServiceProjectLink(structure_models.ServiceProjectLink):
    service = models.ForeignKey(DigitalOceanService)

    class Meta(structure_models.ServiceProjectLink.Meta):
        verbose_name = 'DigitalOcean service project link'
        verbose_name_plural = 'DigitalOcean service project links'

    @classmethod
    def get_url_name(cls):
        return 'digitalocean-spl'


class Region(structure_models.GeneralServiceProperty):
    @classmethod
    def get_url_name(cls):
        return 'digitalocean-region'


@python_2_unicode_compatible
class Image(structure_models.GeneralServiceProperty):
    regions = models.ManyToManyField(Region)
    distribution = models.CharField(max_length=100)
    type = models.CharField(max_length=100)
    is_official = models.BooleanField(default=False, help_text='Is image provided by DigitalOcean')
    min_disk_size = models.PositiveIntegerField(
        null=True, help_text='Minimum disk required for a size to use this image')
    created_at = models.DateTimeField(null=True)

    @property
    def is_ssh_key_mandatory(self):
        MANDATORY = 'Ubuntu', 'FreeBSD', 'CoreOS'
        return self.distribution in MANDATORY

    def __str__(self):
        return '{} {} ({})'.format(self.name, self.distribution, self.type)

    @classmethod
    def get_url_name(cls):
        return 'digitalocean-image'


class Size(structure_models.GeneralServiceProperty):
    regions = models.ManyToManyField(Region)

    cores = models.PositiveSmallIntegerField(help_text='Number of cores in a VM')
    ram = models.PositiveIntegerField(help_text='Memory size in MiB')
    disk = models.PositiveIntegerField(help_text='Disk size in MiB')
    transfer = models.PositiveIntegerField(help_text='Amount of transfer bandwidth in MiB')
    price = models.DecimalField('Hourly price rate', default=0, max_digits=11, decimal_places=5)

    @classmethod
    def get_url_name(cls):
        return 'digitalocean-size'


class Droplet(RuntimeStateMixin, StateMixin, structure_models.VirtualMachineMixin, structure_models.ResourceMixin):
    service_project_link = models.ForeignKey(
        DigitalOceanServiceProjectLink, related_name='droplets', on_delete=models.PROTECT)
    transfer = models.PositiveIntegerField(default=0, help_text='Amount of transfer bandwidth in MiB')
    ip_address = models.GenericIPAddressField(null=True, protocol='IPv4', blank=True)

    @property
    def external_ips(self):
        return [self.ip_address]

    @property
    def internal_ips(self):
        return []

    @classmethod
    def get_url_name(cls):
        return 'digitalocean-droplet'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('nodeconductor_digitalocean', '0002_remove_digitaloceanservice_name'),
    ]

    operations = [
        migrations.RenameField(
            model_name='droplet',
            old_name='external_ips',
            new_name='ip_address',
        ),
        migrations.RemoveField(
            model_name='droplet',
            name='internal_ips',
        ),
    ]



from celery import chain

from nodeconductor.core import executors
from nodeconductor.core.tasks import StateTransitionTask
from nodeconductor.core.models import RuntimeStateMixin
from nodeconductor.core import utils as core_utils

from . import tasks


class DropletCreateExecutor(executors.CreateExecutor):

    @classmethod
    def get_task_signature(cls, droplet, serialized_droplet, **kwargs):
        return chain(
            tasks.SafeBackendMethodTask().si(
                serialized_droplet, 'create_droplet',
                state_transition='begin_creating',
                runtime_state='provisioning',
                success_runtime_state=RuntimeStateMixin.RuntimeStates.ONLINE,
                **kwargs),
            tasks.WaitForActionComplete().s(serialized_droplet).set(countdown=30))


class DropletDeleteExecutor(executors.DeleteExecutor):

    @classmethod
    def get_task_signature(cls, droplet, serialized_droplet, **kwargs):
        if droplet.backend_id:
            return tasks.SafeBackendMethodTask().si(
                serialized_droplet, 'destroy', state_transition='begin_deleting')
        else:
            return StateTransitionTask().si(serialized_droplet, state_transition='begin_deleting')


class DropletStateChangeExecutor(executors.UpdateExecutor):

    @classmethod
    def get_task_signature(cls, droplet, serialized_droplet, method=None, final_state=None, **kwargs):
        return chain(
            tasks.SafeBackendMethodTask().si(
                serialized_droplet, method,
                state_transition='begin_updating',
                success_runtime_state=final_state),
            tasks.WaitForActionComplete().s(serialized_droplet).set(countdown=10))


class DropletResizeExecutor(executors.UpdateExecutor):

    @classmethod
    def get_task_signature(cls, droplet, serialized_droplet, **kwargs):
        size = kwargs.pop('size')
        disk = kwargs.pop('disk')
        return chain(
            tasks.SafeBackendMethodTask().si(
                serialized_droplet, 'resize',
                state_transition='begin_updating',
                runtime_state='resizing',
                success_runtime_state=RuntimeStateMixin.RuntimeStates.ONLINE,
                backend_size_id=size.backend_id,
                disk=disk),
            tasks.WaitForActionComplete().s(serialized_droplet).set(countdown=10),
            tasks.LogDropletResized().si(serialized_droplet, core_utils.serialize_instance(size)))



from nodeconductor.core.permissions import StaffPermissionLogic
from nodeconductor.structure import perms as structure_perms


PERMISSION_LOGICS = (
    ('nodeconductor_digitalocean.DigitalOceanService', structure_perms.service_permission_logic),
    ('nodeconductor_digitalocean.DigitalOceanServiceProjectLink', structure_perms.service_project_link_permission_logic),
    ('nodeconductor_digitalocean.Droplet', structure_perms.resource_permission_logic),
    ('nodeconductor_digitalocean.Image', StaffPermissionLogic(any_permission=True)),
    ('nodeconductor_digitalocean.Region', StaffPermissionLogic(any_permission=True)),
    ('nodeconductor_digitalocean.Size', StaffPermissionLogic(any_permission=True)),
)
